@@ -14,8 +14,8 @@
  * limitations under the License.
  */
 
+use std::fmt::{Debug, Formatter};
 use std::io::{self, Write};
-use std::mem::replace;
 
 use cdchunking::{ChunkerImpl, ZPAQ};
 use serde::{Deserialize, Serialize};
@@ -53,6 +53,12 @@ pub enum Chunking {
 }
 
 impl Chunking {
+    /// A reasonable default value of `Chunking::Fixed`.
+    pub const FIXED: Self = Self::Fixed { size: 1024 * 1024 };
+
+    /// A reasonable default value of `Chunking::Zpaq`.
+    pub const ZPAQ: Self = Self::Zpaq { bits: 18 };
+
     /// Return a chunker for this chunking method.
     pub(super) fn to_chunker(&self) -> Box<dyn ChunkerImpl> {
         match self {
@@ -60,16 +66,6 @@ impl Chunking {
             Chunking::Zpaq { bits } => Box::new(ZPAQ::new(*bits as usize)),
         }
     }
-
-    /// Return a reasonable default value of `Chunking::Fixed`.
-    pub const fn fixed() -> Self {
-        Chunking::Fixed { size: 1024 * 1024 }
-    }
-
-    /// Return a reasonable default value of `Chunking::Zpaq`.
-    pub const fn zpaq() -> Self {
-        Chunking::Zpaq { bits: 18 }
-    }
 }
 
 /// A `ChunkerImpl` which chunks data into fixed-size chunks.
@@ -111,6 +107,15 @@ pub struct IncrementalChunker {
     chunks: Vec<Vec<u8>>,
 }
 
+impl Debug for IncrementalChunker {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("IncrementalChunker")
+            .field("buffer", &self.buffer)
+            .field("chunks", &self.chunks)
+            .finish_non_exhaustive()
+    }
+}
+
 impl IncrementalChunker {
     /// Return a new instance which uses the given `chunker` to determine chunk boundaries.
     pub fn new(chunker: Box<dyn ChunkerImpl>) -> Self {
@@ -127,7 +132,7 @@ impl IncrementalChunker {
     /// still be buffered internally. Calling `flush` will make this method return the remaining
     /// buffered data as a new chunk.
     pub fn chunks(&mut self) -> Vec<Vec<u8>> {
-        replace(&mut self.chunks, Vec::new())
+        std::mem::take(&mut self.chunks)
     }
 
     /// Clear all the data in the chunker.
@@ -155,7 +160,7 @@ impl Write for IncrementalChunker {
                 }
                 Some(index) => {
                     self.buffer.extend_from_slice(&unchunked_data[..index]);
-                    let new_chunk = replace(&mut self.buffer, Vec::new());
+                    let new_chunk = std::mem::take(&mut self.buffer);
                     self.chunks.push(new_chunk);
                     unchunked_data = &unchunked_data[index..];
                     self.chunker.reset();
@@ -166,7 +171,7 @@ impl Write for IncrementalChunker {
 
     fn flush(&mut self) -> io::Result<()> {
         if !self.buffer.is_empty() {
-            let new_chunk = replace(&mut self.buffer, Vec::new());
+            let new_chunk = std::mem::take(&mut self.buffer);
             self.chunks.push(new_chunk);
         }
         self.chunker.reset();
