@@ -23,15 +23,14 @@ use super::packing::Packing;
 
 /// The configuration for an repository.
 ///
-/// This type is used to configure a repository when it is created. Once a repository is created,
-/// the config values provided cannot be changed. This type implements `Default` to provide a
-/// reasonable default configuration.
+/// This type is used to configure a repository when it is created. This type implements `Default`
+/// to provide a reasonable default configuration.
 #[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
 #[non_exhaustive]
 pub struct RepoConfig {
     /// The chunking method to use in the repository.
     ///
-    /// The default value is `Chunking::fixed()`.
+    /// The default value is `Chunking::FIXED`.
     pub chunking: Chunking,
 
     /// The packing method to use in the repository.
@@ -63,7 +62,7 @@ pub struct RepoConfig {
 impl Default for RepoConfig {
     fn default() -> Self {
         RepoConfig {
-            chunking: Chunking::fixed(),
+            chunking: Chunking::FIXED,
             packing: Packing::None,
             compression: Compression::None,
             encryption: Encryption::None,
