@@ -18,15 +18,13 @@ use std::collections::HashMap;
 
 use rmp_serde::from_read;
 use serde::{Deserialize, Serialize};
-use uuid::Uuid;
 
 use super::config::RepoConfig;
 use super::encryption::KeySalt;
-use super::handle::Chunk;
-use super::id_table::IdTable;
+use super::handle::{Chunk, HandleIdTable};
 use super::repository::METADATA_BLOCK_ID;
-use super::state::{ChunkInfo, InstanceInfo, PackIndex};
-use crate::store::{DataStore, OpenStore};
+use super::state::{ChunkInfo, InstanceId, InstanceInfo, PackIndex};
+use crate::store::{BlockId, DataStore, OpenStore};
 
 /// The repository state which is persisted to the data store on each commit.
 #[derive(Debug, Clone, Serialize, Deserialize)]
@@ -35,20 +33,20 @@ pub struct Header {
     pub chunks: HashMap<Chunk, ChunkInfo>,
 
     /// A map of block IDs to their locations in packs.
-    pub packs: HashMap<Uuid, Vec<PackIndex>>,
+    pub packs: HashMap<BlockId, Vec<PackIndex>>,
 
     /// A map of instance IDs to information about each instance.
-    pub instances: HashMap<Uuid, InstanceInfo>,
+    pub instances: HashMap<InstanceId, InstanceInfo>,
 
     /// The table of object handle IDs.
-    pub handle_table: IdTable,
+    pub handle_table: HandleIdTable,
 }
 
 /// Metadata for a repository.
 #[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
 pub struct RepoMetadata {
     /// The unique ID of this repository.
-    pub id: Uuid,
+    pub id: RepoId,
 
     /// The configuration for the repository.
     pub config: RepoConfig,
@@ -60,7 +58,7 @@ pub struct RepoMetadata {
     pub salt: KeySalt,
 
     /// The ID of the chunk which stores the repository header.
-    pub header_id: Uuid,
+    pub header_id: BlockId,
 }
 
 impl RepoMetadata {
@@ -107,24 +105,72 @@ pub fn peek_info(config: &impl OpenStore) -> crate::Result<RepoInfo> {
     peek_info_store(&mut store)
 }
 
+uuid_type! {
+    /// A UUID which uniquely identifies a repository.
+    ///
+    /// This ID is different from the instance ID; this ID is shared between all instances of a
+    /// repository.
+    RepoId
+}
+
 /// Information about a repository.
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct RepoInfo {
-    id: Uuid,
+    id: RepoId,
     config: RepoConfig,
 }
 
 impl RepoInfo {
     /// The unique ID for this repository.
-    ///
-    /// This ID is different from the instance ID; this ID is shared between all instances of a
-    /// repository.
-    pub fn id(&self) -> Uuid {
+    pub fn id(&self) -> RepoId {
         self.id
     }
 
-    /// The configuration used to create this repository.
+    /// The configuration for this repository.
     pub fn config(&self) -> &RepoConfig {
         &self.config
     }
 }
+
+/// Statistics about a repository.
+#[derive(Debug, Clone)]
+pub struct RepoStats {
+    pub(super) apparent_size: u64,
+    pub(super) actual_size: u64,
+    pub(super) repo_size: u64,
+}
+
+impl RepoStats {
+    /// The apparent size of the current instance.
+    ///
+    /// This is the sum of the apparent sizes of all the objects in the current instance of the
+    /// repository, which includes any sparse holes in those objects.
+    pub fn apparent_size(&self) -> u64 {
+        self.apparent_size
+    }
+
+    /// The actual size of the current instance.
+    ///
+    /// This is the actual number of bytes stored in objects in the current instance of the
+    /// repository, which may be smaller than the [`apparent_size`] due to sparse holes in objects
+    /// and deduplication between objects in the current instance.
+    ///
+    /// [`apparent_size`]: crate::repo::RepoStats::apparent_size
+    pub fn actual_size(&self) -> u64 {
+        self.actual_size
+    }
+
+    /// The actual size of the repository.
+    ///
+    /// This is the actual number of bytes stored in objects in all instances of the repository,
+    /// which may be smaller than the sum of the [`actual_size`] of each instance due to
+    /// deduplication between objects in different instances.
+    ///
+    /// This value is not necessarily the same as the number of bytes stored in the backing data
+    /// store, which may be larger or smaller due to compression, encryption, and packing.
+    ///
+    /// [`actual_size`]: crate::repo::RepoStats::actual_size
+    pub fn repo_size(&self) -> u64 {
+        self.repo_size
+    }
+}
