@@ -23,9 +23,8 @@ use serde::de::DeserializeOwned;
 use serde::Serialize;
 
 use super::chunk_store::{ReadChunk, StoreReader, StoreWriter, WriteChunk};
-use super::handle::{chunk_hash, ContentId, ObjectHandle};
+use super::handle::{chunk_hash, ContentId, Extent, ObjectHandle, ObjectStats};
 use super::state::{ExtentLocation, ObjectState, RepoState, SeekPosition};
-use crate::repo::common::handle::Extent;
 
 pub struct ObjectStore {
     repo_state: Arc<RwLock<RepoState>>,
@@ -158,6 +157,36 @@ impl<'a> ObjectInfo<'a> {
             extents: self.handle.extents.clone(),
         })
     }
+
+    /// Return an `ObjectStats` containing statistics about the object.
+    pub fn stats(&self) -> crate::Result<ObjectStats> {
+        if self.object_state.transaction_lock.is_some() {
+            return Err(crate::Error::TransactionInProgress);
+        }
+        let mut current_position = 0u64;
+        let mut actual_size = 0u64;
+        let mut apparent_size = 0u64;
+        let mut holes = Vec::new();
+
+        for extent in &self.handle.extents {
+            match extent {
+                Extent::Chunk(_) => {
+                    actual_size += extent.size();
+                }
+                Extent::Hole { .. } => {
+                    holes.push(current_position..(current_position + extent.size()));
+                }
+            }
+            current_position += extent.size();
+            apparent_size += extent.size();
+        }
+
+        Ok(ObjectStats {
+            apparent_size,
+            actual_size,
+            holes,
+        })
+    }
 }
 
 /// A borrowed value for reading from an object.
@@ -538,12 +567,7 @@ impl<'a> ObjectWriter<'a> {
         if let Some(hole_size) = start_hole_size {
             new_extents.push(Extent::Hole { size: hole_size });
         }
-        new_extents.extend(
-            self.object_state
-                .new_chunks
-                .drain(..)
-                .map(|chunk| Extent::Chunk(chunk)),
-        );
+        new_extents.extend(self.object_state.new_chunks.drain(..).map(Extent::Chunk));
         if let Some(hole_size) = end_hole_size {
             new_extents.push(Extent::Hole { size: hole_size });
         }
