@@ -29,29 +29,31 @@ use super::chunking::Chunking;
 use super::compression::Compression;
 use super::config::RepoConfig;
 use super::encryption::{Encryption, EncryptionKey, KeySalt, ResourceLimit};
-use super::id_table::IdTable;
+use super::handle::HandleIdTable;
 use super::lock::LockTable;
-use super::metadata::{peek_info_store, Header, RepoMetadata};
+use super::metadata::{peek_info_store, Header, RepoId, RepoMetadata};
 use super::open_repo::OpenRepo;
 use super::packing::Packing;
 use super::repository::{KeyRepo, METADATA_BLOCK_ID, VERSION_BLOCK_ID};
-use super::state::RepoState;
+use super::state::{InstanceId, RepoState};
 
 /// The default repository instance ID.
 ///
 /// This is the instance ID that is used by [`OpenOptions`] when an instance isn't specified.
 ///
 /// [`OpenOptions`]: crate::repo::OpenOptions
-pub const DEFAULT_INSTANCE: Uuid = Uuid::from_bytes(hex!("ea978302 bfd8 11ea b92b 031a9ad75c07"));
+pub const DEFAULT_INSTANCE: InstanceId = InstanceId::new(Uuid::from_bytes(hex!(
+    "ea978302 bfd8 11ea b92b 031a9ad75c07"
+)));
 
 /// The current repository format version ID.
 ///
 /// This must be changed any time a backwards-incompatible change is made to the repository
 /// format.
-const VERSION_ID: Uuid = Uuid::from_bytes(hex!("6f1c893c e6a8 11eb a198 b7fa995cc83b"));
+const VERSION_ID: Uuid = Uuid::from_bytes(hex!("44253e72 f08f 11eb a2a3 a701701f8601"));
 
 /// A table of locks on repositories.
-static REPO_LOCKS: Lazy<Mutex<LockTable<Uuid>>> = Lazy::new(|| Mutex::new(LockTable::new()));
+static REPO_LOCKS: Lazy<Mutex<LockTable<RepoId>>> = Lazy::new(|| Mutex::new(LockTable::new()));
 
 /// The mode to use to open a repository.
 #[derive(Debug, PartialEq, Eq, Hash, Clone, Copy)]
@@ -78,42 +80,38 @@ pub enum OpenMode {
 /// [`OpenRepo`].
 ///
 /// # Examples
-/// ```no_run
-/// # #[cfg(feature = "store-directory")] {
+/// ```
 /// use acid_store::repo::{OpenOptions, OpenMode, key::KeyRepo, Chunking, Compression, Encryption, Packing};
-/// use acid_store::store::DirectoryConfig;
+/// use acid_store::store::MemoryConfig;
 ///
-/// let store_config = DirectoryConfig { path: "/path/to/store".into() };
+/// let store_config = MemoryConfig::new();
 /// let mut repo: KeyRepo<String> = OpenOptions::new()
-///     .chunking(Chunking::zpaq())
+///     .chunking(Chunking::ZPAQ)
 ///     .compression(Compression::Lz4 { level: 1 })
 ///     .encryption(Encryption::XChaCha20Poly1305)
-///     .packing(Packing::fixed())
+///     .packing(Packing::FIXED)
 ///     .password(b"password")
 ///     .mode(OpenMode::Create)
 ///     .open(&store_config)
 ///     .unwrap();
-/// # }
 /// ```
-/// ```no_run
-/// # #[cfg(feature = "store-directory")] {
+/// ```
 /// use acid_store::repo::{OpenOptions, OpenMode, key::KeyRepo, Chunking, Compression, Encryption, Packing, RepoConfig};
-/// use acid_store::store::DirectoryConfig;
+/// use acid_store::store::MemoryConfig;
 ///
 /// let mut repo_config = RepoConfig::default();
-/// repo_config.chunking = Chunking::zpaq();
+/// repo_config.chunking = Chunking::ZPAQ;
 /// repo_config.compression = Compression::Lz4 { level: 1 };
 /// repo_config.encryption = Encryption::XChaCha20Poly1305;
-/// repo_config.packing = Packing::fixed();
+/// repo_config.packing = Packing::FIXED;
 ///
-/// let store_config = DirectoryConfig { path: "/path/to/store".into() };
+/// let store_config = MemoryConfig::new();
 /// let mut repo: KeyRepo<String> = OpenOptions::new()
 ///     .config(repo_config)
 ///     .password(b"password")
 ///     .mode(OpenMode::Create)
 ///     .open(&store_config)
 ///     .unwrap();
-/// # }
 /// ```
 ///
 /// [`new`]: crate::repo::OpenOptions::new
@@ -125,7 +123,7 @@ pub struct OpenOptions {
     config: RepoConfig,
     mode: OpenMode,
     password: Option<Vec<u8>>,
-    instance: Uuid,
+    instance: InstanceId,
 }
 
 impl Default for OpenOptions {
@@ -245,7 +243,7 @@ impl OpenOptions {
     /// instances.
     ///
     /// [`DEFAULT_INSTANCE`]: crate::repo::DEFAULT_INSTANCE
-    pub fn instance(&mut self, id: Uuid) -> &mut Self {
+    pub fn instance(&mut self, id: InstanceId) -> &mut Self {
         self.instance = id;
         self
     }
@@ -368,7 +366,7 @@ impl OpenOptions {
         };
 
         // Acquire an exclusive lock on the repository.
-        let id = Uuid::new_v4();
+        let id = Uuid::new_v4().into();
         let lock = REPO_LOCKS
             .lock()
             .unwrap()
@@ -417,7 +415,7 @@ impl OpenOptions {
             chunks: HashMap::new(),
             packs: HashMap::new(),
             instances: HashMap::new(),
-            handle_table: IdTable::new(),
+            handle_table: HandleIdTable::new(),
         };
 
         // Serialize, encode, and write the header to the data store.
@@ -428,7 +426,7 @@ impl OpenOptions {
             .config
             .encryption
             .encrypt(&compressed_header, &master_key);
-        let header_id = Uuid::new_v4();
+        let header_id = Uuid::new_v4().into();
         store
             .write_block(header_id, &encrypted_header)
             .map_err(crate::Error::Store)?;
@@ -436,6 +434,7 @@ impl OpenOptions {
         // Create the repository metadata with the header block references.
         let metadata = RepoMetadata {
             id,
+
             config: self.config.clone(),
             master_key: encrypted_master_key,
             salt,
