@@ -14,10 +14,14 @@
  * limitations under the License.
  */
 
-use uuid::Uuid;
-
 use super::key::Key;
 use super::repository::KeyRepo;
+use super::state::InstanceId;
+
+uuid_type! {
+    /// A UUID which represents the serialized data format of a repository.
+    VersionId
+}
 
 /// A repository which can be opened using [`OpenOptions`].
 ///
@@ -38,7 +42,7 @@ pub trait OpenRepo {
     /// This ID is used to distinguish between different repository types and to detect when the
     /// serialized data format of a repository changes. All backwards-incompatible changes to a
     /// repository's serialized data format must change this value.
-    const VERSION_ID: Uuid;
+    const VERSION_ID: VersionId;
 
     /// Open an existing repository of this type in the backing `repo`.
     ///
@@ -118,12 +122,13 @@ pub trait SwitchInstance {
     ///
     /// # Examples
     /// ```
+    /// # #[cfg(feature = "repo-value")] {
     /// use acid_store::uuid::Uuid;
     /// use acid_store::repo::{SwitchInstance, Commit, OpenMode, OpenOptions, key::KeyRepo, value::ValueRepo};
     /// use acid_store::store::MemoryConfig;
     ///
-    /// let key_instance = Uuid::new_v4();
-    /// let value_instance = Uuid::new_v4();
+    /// let key_instance = Uuid::new_v4().into();
+    /// let value_instance = Uuid::new_v4().into();
     ///
     /// // Open a repository, specifying an instance ID.
     /// let key_repo: KeyRepo<String> = OpenOptions::new()
@@ -137,6 +142,7 @@ pub trait SwitchInstance {
     ///
     /// // Commit both instances of the repository.
     /// value_repo.commit().unwrap();
+    /// # }
     /// ```
     ///
     /// # Errors
@@ -150,14 +156,14 @@ pub trait SwitchInstance {
     /// - `Error::Io`: An I/O error occurred.
     ///
     /// [`OpenOptions::instance`]: crate::repo::OpenOptions::instance
-    fn switch_instance<R>(self, id: Uuid) -> crate::Result<R>
+    fn switch_instance<R>(self, id: InstanceId) -> crate::Result<R>
     where
         R: OpenRepo,
         Self: Sized;
 }
 
 impl<T: OpenRepo> SwitchInstance for T {
-    fn switch_instance<R>(self, id: Uuid) -> crate::Result<R>
+    fn switch_instance<R>(self, id: InstanceId) -> crate::Result<R>
     where
         R: OpenRepo,
         Self: Sized,
