@@ -25,34 +25,36 @@ use rmp_serde::{from_read, to_vec};
 use secrecy::ExposeSecret;
 use uuid::Uuid;
 
-use crate::store::DataStore;
+use crate::store::{BlockId, DataStore};
 
 use super::chunk_store::{
     EncodeBlock, ReadBlock, ReadChunk, StoreReader, StoreState, StoreWriter, WriteBlock,
 };
 use super::commit::Commit;
-use super::encryption::{EncryptionKey, KeySalt};
-use super::handle::{chunk_hash, ObjectHandle, ObjectId};
-use super::id_table::{IdTable, UniqueId};
-use super::key::Key;
-use super::metadata::{Header, RepoInfo};
+use super::encryption::{Encryption, EncryptionKey, KeySalt, ResourceLimit};
+use super::handle::{chunk_hash, HandleId, HandleIdTable, ObjectHandle, ObjectId};
+use super::key::{Key, Keys};
+use super::metadata::{Header, RepoInfo, RepoStats};
 use super::object::Object;
 use super::object_store::{ObjectReader, ObjectWriter};
 use super::open_repo::OpenRepo;
+use super::open_repo::VersionId;
 use super::packing::Packing;
 use super::savepoint::{KeyRestore, RestoreSavepoint, Savepoint};
-use super::state::{InstanceInfo, ObjectState, RepoState};
+use super::state::{InstanceId, InstanceInfo, ObjectState, RepoState};
 
 /// The block ID of the block which stores the repository metadata.
-pub(super) const METADATA_BLOCK_ID: Uuid =
-    Uuid::from_bytes(hex!("8691d360 29c6 11ea 8bc1 2fc8cfe66f33"));
+pub(super) const METADATA_BLOCK_ID: BlockId = BlockId::new(Uuid::from_bytes(hex!(
+    "8691d360 29c6 11ea 8bc1 2fc8cfe66f33"
+)));
 
 /// The block ID of the block which stores the repository format version.
-pub(super) const VERSION_BLOCK_ID: Uuid =
-    Uuid::from_bytes(hex!("cbf28b1c 3550 11ea 8cb0 87d7a14efe10"));
+pub(super) const VERSION_BLOCK_ID: BlockId = BlockId::new(Uuid::from_bytes(hex!(
+    "cbf28b1c 3550 11ea 8cb0 87d7a14efe10"
+)));
 
 /// Return a list of blocks in the data store excluding those used to store metadata.
-fn list_data_blocks(state: &RepoState) -> crate::Result<Vec<Uuid>> {
+fn list_data_blocks(state: &RepoState) -> crate::Result<Vec<BlockId>> {
     let all_blocks = state
         .store
         .lock()
@@ -78,19 +80,19 @@ pub struct KeyRepo<K: Key> {
     pub(super) state: Arc<RwLock<RepoState>>,
 
     /// The instance ID of this repository instance.
-    pub(super) instance_id: Uuid,
+    pub(super) instance_id: InstanceId,
 
     /// A map of object keys to their object handles for the current instance.
     pub(super) objects: HashMap<K, Arc<RwLock<ObjectHandle>>>,
 
     /// A map of instance IDs to information about those instances.
-    pub(super) instances: HashMap<Uuid, InstanceInfo>,
+    pub(super) instances: HashMap<InstanceId, InstanceInfo>,
 
     /// A table of unique IDs of existing handles.
     ///
     /// We use this to determine whether a handle is contained in the repository without actually
     /// storing it.
-    pub(super) handle_table: IdTable,
+    pub(super) handle_table: HandleIdTable,
 
     /// The unique ID for the current transaction.
     ///
@@ -102,7 +104,9 @@ pub struct KeyRepo<K: Key> {
 impl<K: Key> OpenRepo for KeyRepo<K> {
     type Key = K;
 
-    const VERSION_ID: Uuid = Uuid::from_bytes(hex!("989a6a76 9d8b 46b7 9c05 d1c5e0d9471a"));
+    const VERSION_ID: VersionId = VersionId::new(Uuid::from_bytes(hex!(
+        "989a6a76 9d8b 46b7 9c05 d1c5e0d9471a"
+    )));
 
     fn open_repo(repo: KeyRepo<Self::Key>) -> crate::Result<Self>
     where
@@ -125,7 +129,7 @@ impl<K: Key> OpenRepo for KeyRepo<K> {
 
 impl<K: Key> KeyRepo<K> {
     /// Return the `object_id` for the object with the given `handle_id`.
-    fn object_id(&self, handle_id: UniqueId) -> ObjectId {
+    fn object_id(&self, handle_id: HandleId) -> ObjectId {
         let state = self.state.read().unwrap();
         let repo_id = state.metadata.id;
         ObjectId::new(repo_id, self.instance_id, handle_id)
@@ -211,8 +215,8 @@ impl<K: Key> KeyRepo<K> {
     }
 
     /// Return an iterator over all the keys of objects in this repository.
-    pub fn keys(&self) -> impl Iterator<Item = &K> {
-        self.objects.keys()
+    pub fn keys(&self) -> Keys<K> {
+        Keys(self.objects.keys())
     }
 
     /// Copy the object at `source` to `dest`.
@@ -299,7 +303,10 @@ impl<K: Key> KeyRepo<K> {
     ///
     /// This does not write the object map for the current instance before switching to the new
     /// instance.
-    pub(super) fn change_instance<R: OpenRepo>(mut self, instance_id: Uuid) -> crate::Result<R> {
+    pub(super) fn change_instance<R: OpenRepo>(
+        mut self,
+        instance_id: InstanceId,
+    ) -> crate::Result<R> {
         let is_new_instance = !self.instances.contains_key(&instance_id);
 
         let new_objects = if is_new_instance {
@@ -364,7 +371,7 @@ impl<K: Key> KeyRepo<K> {
         let encoded_header = state.encode_data(serialized_header)?;
 
         // Write the new header to a new block.
-        let header_id = Uuid::new_v4();
+        let header_id = Uuid::new_v4().into();
         state
             .store
             .lock()
@@ -405,10 +412,10 @@ impl<K: Key> KeyRepo<K> {
         // put them into the `Header`. This avoids the need to clone them. We'll put them back
         // later.
         let header = Header {
-            chunks: mem::replace(&mut state.chunks, HashMap::new()),
-            packs: mem::replace(&mut state.packs, HashMap::new()),
-            instances: mem::replace(&mut self.instances, HashMap::new()),
-            handle_table: mem::replace(&mut self.handle_table, IdTable::new()),
+            chunks: std::mem::take(&mut state.chunks),
+            packs: std::mem::take(&mut state.packs),
+            instances: std::mem::take(&mut self.instances),
+            handle_table: std::mem::take(&mut self.handle_table),
         };
 
         // Serialize the header so we can write it to the data store.
@@ -548,23 +555,35 @@ impl<K: Key> KeyRepo<K> {
 
     /// Change the password for this repository.
     ///
-    /// This replaces the existing password with `new_password`. Changing the password does not
-    /// require re-encrypting any data. The change does not take effect until [`Commit::commit`] is
-    /// called.
+    /// This replaces the existing password with `new_password`. This also accepts the
+    /// `memory_limit` and the `operations_limit`, which affect the amount of memory and the number
+    /// of computations respectively which will be used by the key derivation function.
+    ///
+    /// Changing the password does not require re-encrypting any data. The change does not take
+    /// effect until [`Commit::commit`] is called.
     ///
     /// If encryption is disabled, this method does nothing.
     ///
     /// [`Commit::commit`]: crate::repo::Commit::commit
-    pub fn change_password(&mut self, new_password: &[u8]) {
+    pub fn change_password(
+        &mut self,
+        new_password: &[u8],
+        memory_limit: ResourceLimit,
+        operations_limit: ResourceLimit,
+    ) {
         let mut state = self.state.write().unwrap();
 
+        if state.metadata.config.encryption == Encryption::None {
+            return;
+        }
+
         let salt = KeySalt::generate();
         let user_key = EncryptionKey::derive(
             new_password,
             &salt,
             state.metadata.config.encryption.key_size(),
-            state.metadata.config.memory_limit,
-            state.metadata.config.operations_limit,
+            memory_limit,
+            operations_limit,
         );
 
         let encrypted_master_key = state
@@ -575,13 +594,60 @@ impl<K: Key> KeyRepo<K> {
 
         state.metadata.salt = salt;
         state.metadata.master_key = encrypted_master_key;
+        state.metadata.config.memory_limit = memory_limit;
+        state.metadata.config.operations_limit = operations_limit;
     }
 
     /// Return this repository's current instance ID.
-    pub fn instance(&self) -> Uuid {
+    pub fn instance(&self) -> InstanceId {
         self.instance_id
     }
 
+    /// Compute statistics about the repository.
+    ///
+    /// The returned `RepoStats` represents the contents of the repository at the time this method
+    /// was called. It is not updated when the repository is modified.
+    pub fn stats(&self) -> RepoStats {
+        let mut apparent_size = 0u64;
+        let mut actual_size = 0u64;
+        let mut repo_size = 0u64;
+
+        // The set of object handle IDs of objects in the current instance.
+        let mut current_instance_handles = HashSet::new();
+
+        // The set of object handle IDs of objects which store metadata and shouldn't count towards
+        // the `repo_size`.
+        let metadata_handles = self
+            .instances
+            .values()
+            .map(|info| info.objects.id)
+            .collect::<HashSet<_>>();
+
+        for handle_lock in self.objects.values() {
+            let handle = handle_lock.read().unwrap();
+            apparent_size += handle.size();
+            current_instance_handles.insert(handle.id);
+        }
+
+        let state = self.state.read().unwrap();
+        for (chunk, info) in state.chunks.iter() {
+            // Only count object inserted by the user in the `repo_size`.
+            if !info.references.is_subset(&metadata_handles) {
+                repo_size += chunk.size as u64;
+            }
+
+            if !info.references.is_disjoint(&current_instance_handles) {
+                actual_size += chunk.size as u64;
+            }
+        }
+
+        RepoStats {
+            apparent_size,
+            actual_size,
+            repo_size,
+        }
+    }
+
     /// Return information about the repository.
     pub fn info(&self) -> RepoInfo {
         self.state.read().unwrap().metadata.to_info()
@@ -810,7 +876,7 @@ impl<K: Key> Commit for KeyRepo<K> {
                     // Temporarily move the pack map into the previous header just so that we can
                     // serialize it. Once we're done, move it back. This avoids needing the clone
                     // the pack map.
-                    previous_header.packs = mem::replace(&mut state.packs, HashMap::new());
+                    previous_header.packs = std::mem::take(&mut state.packs);
                     let serialized_header = to_vec(&previous_header)
                         .expect("Could not serialize the repository header.");
                     mem::swap(&mut previous_header.packs, &mut state.packs);
