@@ -15,39 +15,38 @@
  */
 
 use std::collections::{HashMap, HashSet};
-use std::fmt::{self, Debug, Formatter};
 use std::sync::Mutex;
 
 use cdchunking::ChunkerImpl;
 use serde::{Deserialize, Serialize};
 use uuid::Uuid;
 
-use crate::store::DataStore;
+use crate::store::{BlockId, DataStore};
 
 use super::chunk_store::StoreState;
 use super::chunking::IncrementalChunker;
 use super::encryption::EncryptionKey;
-use super::handle::{Chunk, Extent, ObjectHandle};
-use super::id_table::UniqueId;
+use super::handle::{Chunk, Extent, HandleId, ObjectHandle};
 use super::lock::Lock;
 use super::lock::LockTable;
-use super::metadata::RepoMetadata;
+use super::metadata::{RepoId, RepoMetadata};
+use super::open_repo::VersionId;
 
 /// Information about a chunk in a repository.
-#[derive(Debug, PartialEq, Eq, Clone, Default, Serialize, Deserialize)]
+#[derive(Debug, PartialEq, Eq, Clone, Serialize, Deserialize)]
 pub struct ChunkInfo {
     /// The ID of the block in the data store which stores this chunk.
-    pub block_id: Uuid,
+    pub block_id: BlockId,
 
     /// The IDs of objects which reference this chunk.
-    pub references: HashSet<UniqueId>,
+    pub references: HashSet<HandleId>,
 }
 
 /// The location of a block in a pack.
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct PackIndex {
     /// The UUID of the pack in the data store.
-    pub id: Uuid,
+    pub id: BlockId,
 
     /// The offset from the start of the pack where the block is located.
     pub offset: u32,
@@ -60,7 +59,7 @@ pub struct PackIndex {
 #[derive(Debug)]
 pub struct Pack {
     /// The UUID of this pack in the data store.
-    pub id: Uuid,
+    pub id: BlockId,
 
     /// The data contained in the pack.
     pub buffer: Vec<u8>,
@@ -70,7 +69,7 @@ impl Pack {
     /// Create a new empty pack with the given `pack_size`.
     pub fn new(pack_size: u32) -> Self {
         Pack {
-            id: Uuid::new_v4(),
+            id: Uuid::new_v4().into(),
             buffer: Vec::with_capacity(pack_size as usize),
         }
     }
@@ -87,6 +86,11 @@ impl Pack {
     }
 }
 
+uuid_type! {
+    /// A UUID which uniquely identifies an instance of a repository.
+    InstanceId
+}
+
 /// Information about an instance of a repository.
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct InstanceInfo {
@@ -94,7 +98,7 @@ pub struct InstanceInfo {
     ///
     /// This corresponds to the `OpenRepo::VERSION_ID` of the repository which was created in this
     /// instance.
-    pub version_id: Uuid,
+    pub version_id: VersionId,
 
     /// The object handle used to store the serialized object map.
     ///
@@ -116,19 +120,20 @@ pub struct RepoState {
     pub chunks: HashMap<Chunk, ChunkInfo>,
 
     /// A map of block IDs to their locations in packs.
-    pub packs: HashMap<Uuid, Vec<PackIndex>>,
+    pub packs: HashMap<BlockId, Vec<PackIndex>>,
 
     /// A table used to track current transactions for each object.
-    pub transactions: LockTable<UniqueId>,
+    pub transactions: LockTable<HandleId>,
 
     /// The master encryption key for the repository.
     pub master_key: EncryptionKey,
 
     /// The lock on the repository.
-    pub lock: Lock<Uuid>,
+    pub lock: Lock<RepoId>,
 }
 
 /// A seek position in an object.
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub enum SeekPosition {
     /// The object is empty.
     Empty,
@@ -146,13 +151,13 @@ pub struct ExtentLocation {
     /// The extent itself.
     pub extent: Extent,
 
-    /// The offset of the start of the chunk from the beginning of the object.
+    /// The offset of the start of the extent from the beginning of the object.
     pub start: u64,
 
     /// The offset of the seek position from the beginning of the object.
     pub position: u64,
 
-    /// The index of the chunk in the list of chunks.
+    /// The index of the extent in the list of extents.
     pub index: usize,
 }
 
@@ -164,6 +169,7 @@ impl ExtentLocation {
 }
 
 /// The state associated with an `Object`.
+#[derive(Debug)]
 pub struct ObjectState {
     /// An object responsible for buffering and chunking data which has been written.
     pub chunker: IncrementalChunker,
@@ -189,7 +195,7 @@ pub struct ObjectState {
     pub hole_buffer: Vec<u8>,
 
     /// A lock representing the current transaction if there is one.
-    pub transaction_lock: Option<Lock<UniqueId>>,
+    pub transaction_lock: Option<Lock<HandleId>>,
 
     /// The state for reading and writing blocks to the data store.
     pub store_state: StoreState,
@@ -211,9 +217,3 @@ impl ObjectState {
         }
     }
 }
-
-impl Debug for ObjectState {
-    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
-        write!(f, "ObjectState")
-    }
-}
