@@ -25,10 +25,12 @@ use uuid::Uuid;
 use crate::repo::{
     key::KeyRepo,
     state::{ObjectKey, StateRepo},
-    Commit, OpenRepo, ReadOnlyObject, RepoInfo, RestoreSavepoint, Savepoint,
+    Commit, InstanceId, OpenRepo, ReadOnlyObject, RepoInfo, RepoStats, ResourceLimit,
+    RestoreSavepoint, Savepoint, VersionId,
 };
 
 use super::hash::{HashAlgorithm, BUFFER_SIZE, DEFAULT_ALGORITHM};
+use super::iter::Hashes;
 
 /// The state for a `ContentRepo`.
 #[derive(Debug, Clone, Serialize, Deserialize)]
@@ -62,7 +64,9 @@ pub struct ContentRepo(StateRepo<RepoState>);
 impl OpenRepo for ContentRepo {
     type Key = <StateRepo<RepoState> as OpenRepo>::Key;
 
-    const VERSION_ID: Uuid = Uuid::from_bytes(hex!("91e098e0 cfe3 11eb 8823 77511adc39c8"));
+    const VERSION_ID: VersionId = VersionId::new(Uuid::from_bytes(hex!(
+        "91e098e0 cfe3 11eb 8823 77511adc39c8"
+    )));
 
     fn open_repo(repo: KeyRepo<Self::Key>) -> crate::Result<Self>
     where
@@ -161,8 +165,8 @@ impl ContentRepo {
     }
 
     /// Return an iterator of hashes of all the objects in this repository.
-    pub fn list(&self) -> impl Iterator<Item = &[u8]> {
-        self.0.state().table.keys().map(|hash| hash.as_slice())
+    pub fn hashes(&self) -> Hashes {
+        Hashes(self.0.state().table.keys())
     }
 
     /// Return the hash algorithm used by this repository.
@@ -239,15 +243,30 @@ impl ContentRepo {
     /// See [`KeyRepo::change_password`] for details.
     ///
     /// [`KeyRepo::change_password`]: crate::repo::key::KeyRepo::change_password
-    pub fn change_password(&mut self, new_password: &[u8]) {
-        self.0.change_password(new_password)
+    pub fn change_password(
+        &mut self,
+        new_password: &[u8],
+        memory_limit: ResourceLimit,
+        operations_limit: ResourceLimit,
+    ) {
+        self.0
+            .change_password(new_password, memory_limit, operations_limit)
     }
 
     /// Return this repository's instance ID.
-    pub fn instance(&self) -> Uuid {
+    pub fn instance(&self) -> InstanceId {
         self.0.instance()
     }
 
+    /// Compute statistics about the repository.
+    ///
+    /// See [`KeyRepo::stats`] for details.
+    ///
+    /// [`KeyRepo::stats`]: crate::repo::key::KeyRepo::stats
+    pub fn stats(&self) -> RepoStats {
+        self.0.stats()
+    }
+
     /// Return information about the repository.
     pub fn info(&self) -> RepoInfo {
         self.0.info()
