@@ -40,8 +40,8 @@ use super::object::ObjectTable;
 
 use crate::repo::file::fuse::metadata::to_system_time;
 use crate::repo::file::{
-    repository::EMPTY_PATH, AccessQualifier, Entry, EntryType, FileRepo, UnixMetadata,
-    UnixSpecialType,
+    repository::EMPTY_PATH, AclQualifier, Entry, EntryType, FileMode, FileRepo, UnixMetadata,
+    UnixSpecial,
 };
 use crate::repo::{Commit, RestoreSavepoint};
 
@@ -108,7 +108,7 @@ macro_rules! try_option {
 #[derive(Debug)]
 pub struct FuseAdapter<'a> {
     /// The repository which contains the virtual file system.
-    repo: &'a mut FileRepo<UnixSpecialType, UnixMetadata>,
+    repo: &'a mut FileRepo<UnixSpecial, UnixMetadata>,
 
     /// A table for allocating inodes.
     inodes: InodeTable,
@@ -123,7 +123,7 @@ pub struct FuseAdapter<'a> {
 impl<'a> FuseAdapter<'a> {
     /// Create a new `FuseAdapter` from the given `repo`.
     pub fn new(
-        repo: &'a mut FileRepo<UnixSpecialType, UnixMetadata>,
+        repo: &'a mut FileRepo<UnixSpecial, UnixMetadata>,
         root: &RelativePath,
     ) -> crate::Result<Self> {
         if root == *EMPTY_PATH {
@@ -132,7 +132,7 @@ impl<'a> FuseAdapter<'a> {
 
         let mut inodes = InodeTable::new(root);
 
-        for path in repo.walk(root)? {
+        for path in repo.descendants(root)? {
             inodes.insert(path);
         }
 
@@ -147,7 +147,7 @@ impl<'a> FuseAdapter<'a> {
     /// Get the `FileAttr` for the `entry` with the given `inode`.
     fn entry_attr(
         &mut self,
-        entry: &Entry<UnixSpecialType, UnixMetadata>,
+        entry: &Entry<UnixSpecial, UnixMetadata>,
         inode: u64,
         req: &Request,
     ) -> crate::Result<FileAttr> {
@@ -164,16 +164,16 @@ impl<'a> FuseAdapter<'a> {
             EntryType::Directory => 0,
             EntryType::Special(special) => match special {
                 // The `st_size` of a symlink should be the length of the pathname it contains.
-                UnixSpecialType::SymbolicLink { target } => target.as_os_str().len() as u64,
+                UnixSpecial::Symlink { target } => target.as_os_str().len() as u64,
                 _ => 0,
             },
         };
 
         // The mode returned needs to take into account the ACL mask if it is set, because it
         // affects the group permissions.
-        let mode = match metadata.acl.access.get(&AccessQualifier::Mask) {
-            None => metadata.mode,
-            Some(mask_mode) => (metadata.mode & 0o707) | (mask_mode.bits() << 3),
+        let mode = match metadata.acl.access.get(&AclQualifier::Mask) {
+            None => metadata.mode.bits(),
+            Some(mask_mode) => (metadata.mode.bits() & 0o707) | (mask_mode.bits() << 3),
         };
 
         Ok(FileAttr {
@@ -188,10 +188,10 @@ impl<'a> FuseAdapter<'a> {
                 EntryType::File => fuse::FileType::RegularFile,
                 EntryType::Directory => fuse::FileType::Directory,
                 EntryType::Special(special) => match special {
-                    UnixSpecialType::SymbolicLink { .. } => fuse::FileType::Symlink,
-                    UnixSpecialType::NamedPipe => fuse::FileType::NamedPipe,
-                    UnixSpecialType::BlockDevice { .. } => fuse::FileType::BlockDevice,
-                    UnixSpecialType::CharacterDevice { .. } => fuse::FileType::CharDevice,
+                    UnixSpecial::Symlink { .. } => fuse::FileType::Symlink,
+                    UnixSpecial::NamedPipe => fuse::FileType::NamedPipe,
+                    UnixSpecial::BlockDevice { .. } => fuse::FileType::BlockDevice,
+                    UnixSpecial::CharDevice { .. } => fuse::FileType::CharDevice,
                 },
             },
             perm: mode as u16,
@@ -200,10 +200,10 @@ impl<'a> FuseAdapter<'a> {
             gid: metadata.group,
             rdev: match &entry.kind {
                 EntryType::Special(special) => match special {
-                    UnixSpecialType::BlockDevice { major, minor } => {
+                    UnixSpecial::BlockDevice { major, minor } => {
                         stat::makedev(*major, *minor) as u32
                     }
-                    UnixSpecialType::CharacterDevice { major, minor } => {
+                    UnixSpecial::CharDevice { major, minor } => {
                         stat::makedev(*major, *minor) as u32
                     }
                     _ => NON_SPECIAL_RDEV,
@@ -221,7 +221,7 @@ impl<'a> FuseAdapter<'a> {
     pub fn create_attr(
         &mut self,
         path: RelativePathBuf,
-        entry: &Entry<UnixSpecialType, UnixMetadata>,
+        entry: &Entry<UnixSpecial, UnixMetadata>,
         req: &Request,
     ) -> crate::Result<FileAttr> {
         let entry_inode = self.inodes.insert(path);
@@ -390,7 +390,7 @@ impl<'a> Filesystem for FuseAdapter<'a> {
         let entry_path = try_option!(self.inodes.path(ino), reply, libc::ENOENT);
         let entry = try_result!(self.repo.entry(&entry_path), reply);
         match &entry.kind {
-            EntryType::Special(UnixSpecialType::SymbolicLink { target }) => {
+            EntryType::Special(UnixSpecial::Symlink { target }) => {
                 reply.data(target.as_os_str().as_bytes());
             }
             _ => {
@@ -418,13 +418,13 @@ impl<'a> Filesystem for FuseAdapter<'a> {
         } else if flags.contains(SFlag::S_IFCHR) {
             let major = stat::major(rdev as u64);
             let minor = stat::minor(rdev as u64);
-            EntryType::Special(UnixSpecialType::CharacterDevice { major, minor })
+            EntryType::Special(UnixSpecial::CharDevice { major, minor })
         } else if flags.contains(SFlag::S_IFBLK) {
             let major = stat::major(rdev as u64);
             let minor = stat::minor(rdev as u64);
-            EntryType::Special(UnixSpecialType::BlockDevice { major, minor })
+            EntryType::Special(UnixSpecial::BlockDevice { major, minor })
         } else if flags.contains(SFlag::S_IFIFO) {
-            EntryType::Special(UnixSpecialType::NamedPipe)
+            EntryType::Special(UnixSpecial::NamedPipe)
         } else if flags.contains(SFlag::S_IFSOCK) {
             // Sockets aren't supported by `FileRepo`. `mknod(2)` specifies that `EPERM`
             // should be returned if the file system doesn't support the type of node being
@@ -554,7 +554,7 @@ impl<'a> Filesystem for FuseAdapter<'a> {
         let entry_path = parent_path.join(file_name);
 
         let parent_entry = try_result!(self.repo.entry(&parent_path), reply);
-        let entry = Entry::special(UnixSpecialType::SymbolicLink {
+        let entry = Entry::special(UnixSpecial::Symlink {
             target: link.to_owned(),
         })
         .with_metadata(req)
@@ -614,7 +614,7 @@ impl<'a> Filesystem for FuseAdapter<'a> {
         let source_inode = self.inodes.inode(&source_path).unwrap();
         try_result!(self.objects.commit(source_inode), reply);
         self.objects.close(source_inode);
-        if let Ok(descendants) = self.repo.walk(&source_path) {
+        if let Ok(descendants) = self.repo.descendants(&source_path) {
             for source_descendant in descendants {
                 let descendant_inode = self.inodes.inode(&source_descendant).unwrap();
                 try_result!(self.objects.commit(descendant_inode), reply);
@@ -640,7 +640,7 @@ impl<'a> Filesystem for FuseAdapter<'a> {
 
         // Update the mappings from paths to inodes in the inode table.
         self.inodes.rename(&source_path, dest_path.clone());
-        if let Ok(descendants) = self.repo.walk(&dest_path) {
+        if let Ok(descendants) = self.repo.descendants(&dest_path) {
             for dest_descendant in descendants {
                 let relative_descendant = dest_descendant.strip_prefix(&dest_path).unwrap();
                 let source_descendant = source_path.join(relative_descendant);
@@ -875,7 +875,7 @@ impl<'a> Filesystem for FuseAdapter<'a> {
         }
 
         let mut entries = Vec::new();
-        for child_path in try_result!(self.repo.list(entry_path), reply) {
+        for child_path in try_result!(self.repo.children(entry_path), reply) {
             let file_name = child_path.file_name().unwrap().to_string();
             let inode = self.inodes.inode(&child_path).unwrap();
             let file_type = try_result!(self.repo.entry(&child_path), reply)
@@ -1002,13 +1002,13 @@ impl<'a> Filesystem for FuseAdapter<'a> {
             ACCESS_ACL_XATTR => {
                 let mut permissions = Permissions::from(metadata.clone());
                 try_result!(permissions.update_attr(&attr_name, value), reply);
-                metadata.mode = permissions.mode;
+                metadata.mode = FileMode::from_bits_truncate(permissions.mode);
                 metadata.acl.access = permissions.acl.access;
             }
             DEFAULT_ACL_XATTR => {
                 let mut permissions = Permissions::from(metadata.clone());
                 try_result!(permissions.update_attr(&attr_name, value), reply);
-                metadata.mode = permissions.mode;
+                metadata.mode = FileMode::from_bits_truncate(permissions.mode);
                 metadata.acl.default = permissions.acl.default;
             }
             _ => {}
