@@ -16,8 +16,8 @@
 
 use std::collections::HashSet;
 use std::fmt::Debug;
-use std::fs::{create_dir, create_dir_all, metadata, File, OpenOptions};
-use std::io::{self, copy};
+use std::fs::{create_dir, create_dir_all, metadata};
+use std::io;
 use std::marker::PhantomData;
 use std::path::Path;
 
@@ -28,16 +28,19 @@ use uuid::Uuid;
 use walkdir::WalkDir;
 
 use crate::repo::{
-    key::KeyRepo, state::StateRepo, Commit, Object, OpenRepo, RepoInfo, RestoreSavepoint, Savepoint,
+    key::KeyRepo, state::StateRepo, Commit, InstanceId, Object, OpenRepo, RepoInfo, RepoStats,
+    ResourceLimit, RestoreSavepoint, Savepoint, VersionId,
 };
 
 use super::entry::{Entry, EntryHandle, EntryType, HandleType};
+use super::file::{archive_file, extract_file};
+use super::iter::{Children, Descendants, WalkEntry, WalkPredicate};
 use super::metadata::{FileMetadata, NoMetadata};
 use super::path_tree::PathTree;
-use super::special::{NoSpecialType, SpecialType};
+use super::special::{NoSpecial, SpecialType};
 #[cfg(all(any(unix, doc), feature = "fuse-mount"))]
 use {
-    super::fuse::FuseAdapter, super::metadata::UnixMetadata, super::special::UnixSpecialType,
+    super::fuse::FuseAdapter, super::metadata::UnixMetadata, super::special::UnixSpecial,
     std::ffi::OsStr,
 };
 
@@ -50,7 +53,7 @@ type RepoState = PathTree<EntryHandle>;
 ///
 /// See [`crate::repo::file`] for more information.
 #[derive(Debug)]
-pub struct FileRepo<S = NoSpecialType, M = NoMetadata>(
+pub struct FileRepo<S = NoSpecial, M = NoMetadata>(
     pub(super) StateRepo<RepoState>,
     PhantomData<(S, M)>,
 )
@@ -65,7 +68,9 @@ where
 {
     type Key = <StateRepo<RepoState> as OpenRepo>::Key;
 
-    const VERSION_ID: Uuid = Uuid::from_bytes(hex!("075bc448 d03c 11eb a9e8 43df5f0eb6a0"));
+    const VERSION_ID: VersionId = VersionId::new(Uuid::from_bytes(hex!(
+        "075bc448 d03c 11eb a9e8 43df5f0eb6a0"
+    )));
 
     fn open_repo(repo: KeyRepo<Self::Key>) -> crate::Result<Self>
     where
@@ -170,16 +175,16 @@ where
     /// # #[cfg(feature = "file-metadata")] {
     /// # use std::path::Path;
     /// # use acid_store::repo::{OpenOptions, OpenMode};
-    /// # use acid_store::repo::file::{FileRepo, Entry, RelativePath, UnixSpecialType};
+    /// # use acid_store::repo::file::{FileRepo, Entry, RelativePath, UnixSpecial};
     /// # use acid_store::store::{MemoryStore, MemoryConfig};
     /// #
-    /// # let mut repo: FileRepo<UnixSpecialType> = OpenOptions::new()
+    /// # let mut repo: FileRepo<UnixSpecial> = OpenOptions::new()
     /// #    .mode(OpenMode::CreateNew)
     /// #    .open(&MemoryConfig::new())
     /// #    .unwrap();
     /// #
     /// let entry_path = RelativePath::new("link");
-    /// let symbolic_link = UnixSpecialType::SymbolicLink {
+    /// let symbolic_link = UnixSpecial::Symlink {
     ///     target: Path::new("target").to_owned()
     /// };
     /// repo.create(entry_path, &Entry::special(symbolic_link)).unwrap();
@@ -283,7 +288,7 @@ where
             return Err(crate::Error::InvalidPath);
         }
 
-        match self.0.state().list(&path) {
+        match self.0.state().children(&path) {
             Some(mut children) => {
                 if children.next().is_some() {
                     return Err(crate::Error::NotEmpty);
@@ -525,7 +530,7 @@ where
 
         // Get the destination paths for each path in the path table and insert them into the
         // destination tree.
-        for (path, source_handle) in self.0.state().walk(source.as_ref()).unwrap() {
+        for (path, source_handle) in self.0.state().descendants(source.as_ref()).unwrap() {
             let relative_path = path.strip_prefix(&source).unwrap();
             let dest_tree_path = dest_tree_root.join(relative_path);
             dest_tree.insert(dest_tree_path, *source_handle);
@@ -542,30 +547,41 @@ where
         Ok(())
     }
 
-    /// Return an iterator of paths which are children of `parent`.
+    /// Verify that `path` has descendants.
+    fn verify_has_descendants(&self, parent: &RelativePath) -> crate::Result<()> {
+        if parent == *EMPTY_PATH {
+            return Ok(());
+        }
+
+        let entry_handle = self
+            .0
+            .state()
+            .get(parent.as_ref())
+            .ok_or(crate::Error::NotFound)?;
+
+        if !matches!(entry_handle.kind, HandleType::Directory) {
+            return Err(crate::Error::NotDirectory);
+        }
+
+        Ok(())
+    }
+
+    /// Return an iterator of paths which are immediate children of `parent`.
     ///
     /// The given `parent` may be an empty path, in which case the paths of top-level entries are
     /// returned.
     ///
+    /// The `parent` path is not included in the returned iterator.
+    ///
     /// # Errors
     /// - `Error::NotFound`: The given `parent` does not exist.
     /// - `Error::NotDirectory`: The given `parent` is not a directory.
-    pub fn list<'a>(
+    pub fn children<'a>(
         &'a self,
         parent: impl AsRef<RelativePath> + 'a,
-    ) -> crate::Result<impl Iterator<Item = RelativePathBuf> + 'a> {
-        if parent.as_ref() != *EMPTY_PATH {
-            let entry_handle = self
-                .0
-                .state()
-                .get(parent.as_ref())
-                .ok_or(crate::Error::NotFound)?;
-            if !matches!(entry_handle.kind, HandleType::Directory) {
-                return Err(crate::Error::NotDirectory);
-            }
-        }
-
-        Ok(self.0.state().list(parent).unwrap().map(|(path, _)| path))
+    ) -> crate::Result<Children<'a>> {
+        self.verify_has_descendants(parent.as_ref())?;
+        Ok(Children(self.0.state().children(parent).unwrap()))
     }
 
     /// Return an iterator of paths which are descendants of `parent`.
@@ -573,28 +589,106 @@ where
     /// The given `parent` may be an empty path, in which case the paths of all entries in the
     /// repository are returned.
     ///
+    /// The `parent` path is not included in the returned iterator.
+    ///
     /// The returned iterator yields paths in depth-first order, meaning that a path will always
     /// come before its children.
     ///
     /// # Errors
     /// - `Error::NotFound`: The given `parent` does not exist.
     /// - `Error::NotDirectory`: The given `parent` is not a directory.
-    pub fn walk<'a>(
+    pub fn descendants<'a>(
         &'a self,
         parent: impl AsRef<RelativePath> + 'a,
-    ) -> crate::Result<impl Iterator<Item = RelativePathBuf> + 'a> {
-        if parent.as_ref() != *EMPTY_PATH {
-            let entry_handle = self
-                .0
-                .state()
-                .get(parent.as_ref())
-                .ok_or(crate::Error::NotFound)?;
-            if !matches!(entry_handle.kind, HandleType::Directory) {
-                return Err(crate::Error::NotDirectory);
-            }
-        }
+    ) -> crate::Result<Descendants<'a>> {
+        self.verify_has_descendants(parent.as_ref())?;
+        Ok(Descendants(self.0.state().descendants(parent).unwrap()))
+    }
 
-        Ok(self.0.state().walk(parent).unwrap().map(|(path, _)| path))
+    /// Walk through the tree of descendants of `parent`.
+    ///
+    /// This method accepts a `visitor` which is passed a `WalkEntry` for each entry in the tree and
+    /// returns a `WalkPredicate`. If the `visitor` returns `WalkPredicate::Stop` at any point, this
+    /// method returns early with the wrapped value. Otherwise, this method returns `Ok(None)`.
+    ///
+    /// The given `parent` may be an empty path, in which case all entries in the repository are
+    /// visited
+    ///
+    /// The `parent` path is not visited.
+    ///
+    /// This method visits entries in depth-first order, meaning that an entry will always be
+    /// visited before its children.
+    ///
+    /// # Examples
+    /// Find all the paths of empty file entries in the repository.
+    /// ```no_run
+    /// # use acid_store::repo::{OpenOptions, OpenMode};
+    /// # use acid_store::repo::file::{WalkPredicate, FileRepo};
+    /// # use acid_store::store::MemoryConfig;
+    /// #
+    /// # let repo: FileRepo = OpenOptions::new()
+    /// #    .mode(OpenMode::CreateNew)
+    /// #    .open(&MemoryConfig::new())
+    /// #    .unwrap();
+    /// #
+    /// let mut empty_files = Vec::new();
+    /// repo.walk::<(), _, _>("", |entry| {
+    ///     if let Some(object) = entry.open() {
+    ///         if object.size().unwrap() == 0 {
+    ///             empty_files.push(entry.into_path());
+    ///         }
+    ///     }
+    ///     WalkPredicate::Continue
+    /// });
+    /// ```
+    ///
+    /// Search two directory trees for the path of a symbolic link with a given target.
+    /// ```no_run
+    /// # use acid_store::repo::{OpenOptions, OpenMode};
+    /// # use acid_store::repo::file::{UnixSpecial, EntryType, WalkPredicate, FileRepo};
+    /// # use acid_store::store::MemoryConfig;
+    /// #
+    /// # let repo: FileRepo = OpenOptions::new()
+    /// #    .mode(OpenMode::CreateNew)
+    /// #    .open(&MemoryConfig::new())
+    /// #    .unwrap();
+    /// #
+    /// let symlink = UnixSpecial::Symlink { target: "/home/lostatc/target".into() };
+    /// let mut result = repo.walk("", |entry| {
+    ///     if !entry.path().starts_with("first") && !entry.path().starts_with("second") {
+    ///         return WalkPredicate::SkipDescendants;
+    ///     }
+    ///
+    ///     match entry.entry().map(|entry| entry.kind) {
+    ///         Ok(EntryType::Special(symlink)) => WalkPredicate::Stop(Ok(entry.into_path())),
+    ///         Err(error) => WalkPredicate::Stop(Err(error)),
+    ///         _ => WalkPredicate::Continue,
+    ///     }
+    /// });
+    /// ```
+    ///
+    /// # Errors
+    /// - `Error::NotFound`: The given `parent` does not exist.
+    /// - `Error::NotDirectory`: The given `parent` is not a directory.
+    pub fn walk<R, P, F>(&self, parent: P, mut visitor: F) -> crate::Result<Option<R>>
+    where
+        P: AsRef<RelativePath>,
+        F: FnMut(WalkEntry<S, M>) -> WalkPredicate<R>,
+    {
+        self.verify_has_descendants(parent.as_ref())?;
+
+        Ok(self
+            .0
+            .state()
+            .walk(parent, |walk_entry| {
+                visitor(WalkEntry {
+                    path: walk_entry.path,
+                    handle: *walk_entry.value,
+                    depth: walk_entry.depth,
+                    repo: &self,
+                })
+            })
+            .unwrap())
     }
 
     /// Copy a file from the file system into the repository.
@@ -604,6 +698,9 @@ where
     /// The `source` file's metadata will be copied to the `dest` entry according to the selected
     /// [`FileMetadata`] implementation.
     ///
+    /// If `source` is a sparse file, this method will attempt to efficiently copy any sparse holes
+    /// in the file to the [`Object`] in the repository, creating a sparse object.
+    ///
     /// # Errors
     /// - `Error::NotFound`: The parent of `dest` does not exist.
     /// - `Error::NotDirectory`: The parent of `dest` is not a directory entry.
@@ -616,6 +713,7 @@ where
     /// - `Error::Io`: An I/O error occurred.
     ///
     /// [`FileMetadata`]: crate::repo::file::FileMetadata
+    /// [`Object`]: crate::repo::Object
     pub fn archive(
         &mut self,
         source: impl AsRef<Path>,
@@ -641,7 +739,7 @@ where
 
         let entry = Entry {
             kind: file_type,
-            metadata: Some(M::from_file(source.as_ref())?),
+            metadata: M::from_file(source.as_ref())?,
         };
 
         self.create(&dest, &entry)?;
@@ -650,9 +748,7 @@ where
         let entry_handle = self.0.state().get(dest.as_ref()).unwrap();
         if let HandleType::File(object_id) = entry_handle.kind {
             let mut object = self.0.object(object_id).unwrap();
-            let mut file = File::open(&source)?;
-            copy(&mut file, &mut object)?;
-            object.commit()?;
+            archive_file(&mut object, source.as_ref())?;
         }
 
         Ok(())
@@ -664,8 +760,12 @@ where
     /// directory, this is the same as calling [`archive`]. If one of the files in the tree is not a
     /// regular file, directory, or supported special file, it is skipped.
     ///
-    /// The `source` file's metadata will be copied to the `dest` entry according to the selected
-    /// [`FileMetadata`] implementation.
+    /// The metadata of the files in the `source` tree will be copied to the `dest` entries
+    /// according to the selected [`FileMetadata`] implementation.
+    ///
+    /// If a file in the `source` tree is a sparse file, this method will attempt to efficiently
+    /// copy any sparse holes in the file to the [`Object`] in the repository, creating a sparse
+    /// object.
     ///
     /// # Errors
     /// - `Error::NotFound`: The parent of `dest` does not exist.
@@ -709,6 +809,9 @@ where
     /// The `source` entry's metadata will be copied to the `dest` file according to the selected
     /// [`FileMetadata`] implementation.
     ///
+    /// If `source` is a sparse object, this method will attempt to efficiently copy any sparse
+    /// holes in the [`Object`] to the file in the file system, creating a sparse file.
+    ///
     /// # Errors
     /// - `Error::InvalidPath`: The given `source` path is empty.
     /// - `Error::NotFound`: The `source` entry does not exist.
@@ -719,6 +822,7 @@ where
     /// - `Error::Io`: An I/O error occurred.
     ///
     /// [`FileMetadata`]: crate::repo::file::FileMetadata
+    /// [`Object`]: crate::repo::Object
     pub fn extract(
         &self,
         source: impl AsRef<RelativePath>,
@@ -743,11 +847,7 @@ where
         match entry.kind {
             EntryType::File => {
                 let mut object = self.open(source.as_ref()).unwrap();
-                let mut file = OpenOptions::new()
-                    .write(true)
-                    .create_new(true)
-                    .open(&dest)?;
-                copy(&mut object, &mut file)?;
+                extract_file(&mut object, dest.as_ref())?;
             }
             EntryType::Directory => {
                 create_dir(&dest)?;
@@ -770,8 +870,12 @@ where
     /// If `source` is a directory, this also copies its descendants. If `source` is not a
     /// directory, this is the same as calling [`extract`].
     ///
-    /// The `source` entry's metadata will be copied to the `dest` file according to the selected
-    /// [`FileMetadata`] implementation.
+    /// The metadata of the entries in the `source` tree will be copied to the `dest` files
+    /// according to the selected [`FileMetadata`] implementation.
+    ///
+    /// If an entry in the `source` tree is a sparse object, this method will attempt to efficiently
+    /// copy any sparse holes in the [`Object`] to the file in the file system, creating a sparse
+    /// file.
     ///
     /// # Errors
     /// - `Error::InvalidPath`: The given `source` path is empty.
@@ -784,6 +888,7 @@ where
     ///
     /// [`extract`]: crate::repo::file::FileRepo::extract
     /// [`FileMetadata`]: crate::repo::file::FileMetadata
+    /// [`Object`]: crate::repo::Object
     pub fn extract_tree(
         &self,
         source: impl AsRef<RelativePath>,
@@ -792,7 +897,7 @@ where
         let relative_descendants = self
             .0
             .state()
-            .walk(&source)
+            .descendants(&source)
             .ok_or(crate::Error::NotFound)?
             .map(|(path, _)| path.strip_prefix(&source).unwrap().to_owned());
 
@@ -829,7 +934,7 @@ where
         Ok(self
             .0
             .state()
-            .walk(&*EMPTY_PATH)
+            .descendants(&*EMPTY_PATH)
             .unwrap()
             .filter(|(_, entry_handle)| {
                 let entry_corrupt = corrupt_keys.contains(&entry_handle.entry);
@@ -857,15 +962,30 @@ where
     /// See [`KeyRepo::change_password`] for details.
     ///
     /// [`KeyRepo::change_password`]: crate::repo::key::KeyRepo::change_password
-    pub fn change_password(&mut self, new_password: &[u8]) {
-        self.0.change_password(new_password);
+    pub fn change_password(
+        &mut self,
+        new_password: &[u8],
+        memory_limit: ResourceLimit,
+        operations_limit: ResourceLimit,
+    ) {
+        self.0
+            .change_password(new_password, memory_limit, operations_limit);
     }
 
     /// Return this repository's instance ID.
-    pub fn instance(&self) -> Uuid {
+    pub fn instance(&self) -> InstanceId {
         self.0.instance()
     }
 
+    /// Compute statistics about the repository.
+    ///
+    /// See [`KeyRepo::stats`] for details.
+    ///
+    /// [`KeyRepo::stats`]: crate::repo::key::KeyRepo::stats
+    pub fn stats(&self) -> RepoStats {
+        self.0.stats()
+    }
+
     /// Return information about the repository.
     pub fn info(&self) -> RepoInfo {
         self.0.info()
@@ -914,7 +1034,7 @@ const DEFAULT_FUSE_MOUNT_OPTS: &[&str] = &["-o", "default_permissions"];
 
 #[cfg(all(any(unix, doc), feature = "fuse-mount"))]
 #[cfg_attr(docsrs, doc(cfg(all(unix, feature = "fuse-mount"))))]
-impl FileRepo<UnixSpecialType, UnixMetadata> {
+impl FileRepo<UnixSpecial, UnixMetadata> {
     /// Mount the `FileRepo` as a FUSE file system.
     ///
     /// This accepts the path of the `root` entry in the repository which will be mounted in the
