@@ -24,10 +24,12 @@ use serde::de::DeserializeOwned;
 use serde::Serialize;
 use uuid::Uuid;
 
+use super::iter::Keys;
 use crate::repo::{
     key::{Key, KeyRepo},
     state::{ObjectKey, StateRepo},
-    Commit, OpenRepo, RepoInfo, RestoreSavepoint, Savepoint,
+    Commit, InstanceId, OpenRepo, RepoInfo, RepoStats, ResourceLimit, RestoreSavepoint, Savepoint,
+    VersionId,
 };
 
 type RepoState<K> = HashMap<K, ObjectKey>;
@@ -41,7 +43,9 @@ pub struct ValueRepo<K: Key>(StateRepo<RepoState<K>>);
 impl<K: Key> OpenRepo for ValueRepo<K> {
     type Key = <StateRepo<RepoState<K>> as OpenRepo>::Key;
 
-    const VERSION_ID: Uuid = Uuid::from_bytes(hex!("4db4c84c cfc7 11eb 9e06 77121c3277f7"));
+    const VERSION_ID: VersionId = VersionId::new(Uuid::from_bytes(hex!(
+        "4db4c84c cfc7 11eb 9e06 77121c3277f7"
+    )));
 
     fn open_repo(repo: KeyRepo<Self::Key>) -> crate::Result<Self>
     where
@@ -140,8 +144,8 @@ impl<K: Key> ValueRepo<K> {
     }
 
     /// Return an iterator of all the keys in this repository.
-    pub fn keys(&self) -> impl Iterator<Item = &K> {
-        self.0.state().keys()
+    pub fn keys(&self) -> Keys<K> {
+        Keys(self.0.state().keys())
     }
 
     /// Copy the value at `source` to `dest`.
@@ -198,15 +202,30 @@ impl<K: Key> ValueRepo<K> {
     /// See [`KeyRepo::change_password`] for details.
     ///
     /// [`KeyRepo::change_password`]: crate::repo::key::KeyRepo::change_password
-    pub fn change_password(&mut self, new_password: &[u8]) {
-        self.0.change_password(new_password);
+    pub fn change_password(
+        &mut self,
+        new_password: &[u8],
+        memory_limit: ResourceLimit,
+        operations_limit: ResourceLimit,
+    ) {
+        self.0
+            .change_password(new_password, memory_limit, operations_limit);
     }
 
     /// Return this repository's instance ID.
-    pub fn instance(&self) -> Uuid {
+    pub fn instance(&self) -> InstanceId {
         self.0.instance()
     }
 
+    /// Compute statistics about the repository.
+    ///
+    /// See [`KeyRepo::stats`] for details.
+    ///
+    /// [`KeyRepo::stats`]: crate::repo::key::KeyRepo::stats
+    pub fn stats(&self) -> RepoStats {
+        self.0.stats()
+    }
+
     /// Return information about the repository.
     pub fn info(&self) -> RepoInfo {
         self.0.info()
