@@ -28,13 +28,15 @@ use uuid::Uuid;
 use crate::repo::key::KeyRepo;
 use crate::repo::state::StateRepo;
 use crate::repo::{
-    key::Key, Commit, Object, OpenRepo, ReadOnlyObject, RepoInfo, RestoreSavepoint, Savepoint,
+    key::Key, Commit, InstanceId, Object, OpenRepo, ReadOnlyObject, RepoInfo, RepoStats,
+    ResourceLimit, RestoreSavepoint, Savepoint, VersionId,
 };
 
 use super::info::{KeyInfo, Version, VersionInfo};
+use super::iter::{Keys, Versions};
 
 /// The state for a `VersionRepo`.
-type RepoState<K> = HashMap<K, KeyInfo>;
+pub type RepoState<K> = HashMap<K, KeyInfo>;
 
 /// The ID of the first version of an object.
 const INITIAL_VERSION_ID: u32 = 1;
@@ -48,7 +50,9 @@ pub struct VersionRepo<K: Key>(StateRepo<RepoState<K>>);
 impl<K: Key> OpenRepo for VersionRepo<K> {
     type Key = <StateRepo<RepoState<K>> as OpenRepo>::Key;
 
-    const VERSION_ID: Uuid = Uuid::from_bytes(hex!("41a76832 cfc4 11eb ad05 93c1b714dd17"));
+    const VERSION_ID: VersionId = VersionId::new(Uuid::from_bytes(hex!(
+        "41a76832 cfc4 11eb ad05 93c1b714dd17"
+    )));
 
     fn open_repo(repo: KeyRepo<Self::Key>) -> crate::Result<Self>
     where
@@ -138,8 +142,8 @@ impl<K: Key> VersionRepo<K> {
     }
 
     /// Return an iterator of all the keys in this repository.
-    pub fn keys(&self) -> impl Iterator<Item = &K> {
-        self.0.state().keys()
+    pub fn keys(&self) -> Keys<K> {
+        Keys(self.0.state().keys())
     }
 
     /// Create a new version of the given `key` and return it.
@@ -255,17 +259,18 @@ impl<K: Key> VersionRepo<K> {
     ///
     /// The versions are sorted by their version ID, which corresponds to the order they were
     /// created in.
-    pub fn versions<'a, Q>(&'a self, key: &Q) -> Option<impl Iterator<Item = Version> + 'a>
+    pub fn versions<Q>(&self, key: &Q) -> Option<Versions>
     where
         K: Borrow<Q>,
         Q: Hash + Eq + ?Sized,
     {
-        let versions = &self.0.state().get(key)?.versions;
-        Some(versions.iter().map(move |(id, info)| Version {
-            id: *id,
-            created: info.created,
-            content_id: self.0.object(info.id).unwrap().content_id().unwrap(),
-        }))
+        let versions = self.0.state().get(key)?.versions.clone().into_iter();
+        Some(Versions {
+            versions,
+            id_factory: Box::new(move |object_key| {
+                self.0.object(object_key).unwrap().content_id().unwrap()
+            }),
+        })
     }
 
     /// Replace the current version of `key` with the version with the given `version_id`.
@@ -315,15 +320,30 @@ impl<K: Key> VersionRepo<K> {
     /// See [`KeyRepo::change_password`] for details.
     ///
     /// [`KeyRepo::change_password`]: crate::repo::key::KeyRepo::change_password
-    pub fn change_password(&mut self, new_password: &[u8]) {
-        self.0.change_password(new_password);
+    pub fn change_password(
+        &mut self,
+        new_password: &[u8],
+        memory_limit: ResourceLimit,
+        operations_limit: ResourceLimit,
+    ) {
+        self.0
+            .change_password(new_password, memory_limit, operations_limit);
     }
 
     /// Return this repository's instance ID.
-    pub fn instance(&self) -> Uuid {
+    pub fn instance(&self) -> InstanceId {
         self.0.instance()
     }
 
+    /// Compute statistics about the repository.
+    ///
+    /// See [`KeyRepo::stats`] for details.
+    ///
+    /// [`KeyRepo::stats`]: crate::repo::key::KeyRepo::stats
+    pub fn stats(&self) -> RepoStats {
+        self.0.stats()
+    }
+
     /// Return information about the repository.
     pub fn info(&self) -> RepoInfo {
         self.0.info()
